#!/usr/bin/env python

##############################################################################
#	This software was developed by the University of Tennessee as part of the
#	Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
#	project funded by the US National Science Foundation.
#
#	If you use DANSE applications to do scientific research that leads to
#	publication, we ask that you acknowledge the use of the software with the
#	following sentence:
#
#	"This work benefited from DANSE software developed under NSF award DMR-0520547."
#
#	copyright 2008, University of Tennessee
##############################################################################


""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY ..\c_extensions\bcc.h
         AND RE-RUN THE GENERATOR SCRIPT

"""

from sans.models.BaseComponent import BaseComponent
from sans_extension.c_models import CBCCrystalModel
import copy    
    
class BCCrystalModel(CBCCrystalModel, BaseComponent):
    """ 
    Class that evaluates a BCCrystalModel model. 
    This file was auto-generated from ..\c_extensions\bcc.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         scale           = 1.0 
         dnn             = 220.0 [A]
         d_factor        = 0.06 
         radius          = 40.0 [A]
         sldSph          = 3e-006 [1/A^(2)]
         sldSolv         = 6.3e-006 [1/A^(2)]
         background      = 0.0 [1/cm]
         theta           = 0.0 [rad]
         phi             = 0.0 [rad]
         psi             = 0.0 [rad]

    """
        
    def __init__(self):
        """ Initialization """
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        CBCCrystalModel.__init__(self)
        
        ## Name of the model
        self.name = "BCCrystalModel"
        ## Model description
        self.description ="""P(q)=(scale/Vp)*V_lattice*P(q)*Z(q)+bkg where scale is the volume
		fraction of sphere,
		Vp = volume of the primary particle,
		V_lattice = volume correction for
		for the crystal structure,
		P(q)= form factor of the sphere (normalized),
		Z(q)= paracrystalline structure factor
		for a face centered cubic structure.
		[Body Centered Cubic ParaCrystal Model]
		Parameters;
		scale: volume fraction of spheres
		bkg:background, R: radius of sphere
		dnn: Nearest neighbor distance
		d_factor: Paracrystal distortion factor
		radius: radius of the spheres
		sldSph: SLD of the sphere
		sldSolv: SLD of the solvent
		"""
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['scale'] = ['', None, None]
        self.details['dnn'] = ['[A]', None, None]
        self.details['d_factor'] = ['', None, None]
        self.details['radius'] = ['[A]', None, None]
        self.details['sldSph'] = ['[1/A^(2)]', None, None]
        self.details['sldSolv'] = ['[1/A^(2)]', None, None]
        self.details['background'] = ['[1/cm]', None, None]
        self.details['theta'] = ['[rad]', None, None]
        self.details['phi'] = ['[rad]', None, None]
        self.details['psi'] = ['[rad]', None, None]

        ## fittable parameters
        self.fixed=['radius.width', 'phi.width', 'psi.width', 'theta.width']
        
        ## non-fittable parameters
        self.non_fittable=[]
        
        ## parameters with orientation
        self.orientation_params =['phi', 'psi', 'theta', 'phi.width', 'psi.width', 'theta.width']
   
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(BCCrystalModel())   
        
    def __getstate__(self):
        """
        return object state for pickling and copying
        """
        model_state = {'params': self.params, 'dispersion': self.dispersion, 'log': self.log}
        
        return self.__dict__, model_state
        
    def __setstate__(self, state):
        """
        create object from pickled state
        
        :param state: the state of the current model
        
        """
        
        self.__dict__, model_state = state
        self.params = model_state['params']
        self.dispersion = model_state['dispersion']
        self.log = model_state['log']
       	
   
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        
        return CBCCrystalModel.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        
        return CBCCrystalModel.runXY(self, x)
        
    def evalDistribution(self, x=[]):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CBCCrystalModel.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CBCCrystalModel.calculate_ER(self)
        
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CBCCrystalModel.set_dispersion(self, parameter, dispersion.cdisp)
        
   
# End of file
